import subprocess
import yaml
from datetime import datetime


def get_git_history(reversed=False):
    if reversed:
        cmd = ['git', 'log', '--pretty=format:%h|%s|%ad', '--date=iso', '--reverse']
    else:
        cmd = ['git', 'log', '--pretty=format:%h|%s|%ad', '--date=iso']
    result = subprocess.run(cmd, capture_output=True, text=True)
    return [line.split('|') for line in result.stdout.splitlines()]


def get_version_changes(config_file='build_config.yaml'):
    history = get_git_history(reversed=True)
    version_changes = []
    last_version = None
    
    for commit in history:
        try:
            # Check if build_config.yaml exists in this commit
            exists = subprocess.run(['git', 'cat-file', '-e', f'{commit[0]}:{config_file}'],
                                  capture_output=True)
            if exists.returncode != 0:
                continue
                
            # Get the actual build_config.yaml content
            content = subprocess.run(['git', 'show', f'{commit[0]}:{config_file}'],
                                   capture_output=True, text=True)
            config = yaml.safe_load(content.stdout)
            current_version = config.get('project', {}).get('version')
            
            # Only add if version changed and is valid
            if current_version and current_version != last_version:
                version_changes.append((commit[0], current_version, commit[2], commit[1]))
                last_version = current_version
        except Exception:
            continue
    return version_changes


def generate_changelog(config_file='build_config.yaml'):
    version_changes = get_version_changes(config_file)
    print(f"Found {len(version_changes)} version changes")
    for version_change in version_changes:
        print(f"{version_change[1]} ({version_change[2]}) Hash: {version_change[0]} - {version_change[-1]}")
    commits = get_git_history(reversed=False)
    
    changelog = '# Changelog for Papagayo-NG\n\n'
    excluded_parts = ['Merge branch', 'webpage', 'twitter', 'Facebook', 'Update Version to', 'C++', 'Webpage']
    
    # Iterate through version changes in reverse order (newest first)
    for i, (commit_hash, version, date, commit_message) in enumerate(version_changes):
        start_date = date
        end_date = version_changes[i+1][2] if i+1 < len(version_changes) else datetime.now().strftime('%Y-%m-%d %H:%M:%S%z')
        
        simplified_start_date_for_changelog = start_date.split(' ')[0]
        simplified_end_date_for_changelog = end_date.split(' ')[0]

        changelog += f'## Version {version} ({simplified_start_date_for_changelog})\n\n'
        last_version = version_changes[i+1][1] if i+1 < len(version_changes) else None
        # changelog += f'*Version updated in commit {commit_hash[:7]} (from {last_version} to {version}): {commit_message}*\n\n'
        # Add commits for this version period
        for commit in commits:
            commit_date = commit[2]
            if start_date <= commit_date < end_date:
                for part in excluded_parts:
                    if part in commit[1]:
                        break
                else:
                    if i == len(version_changes) - 1:
                        if commit_date > start_date:
                            print(f"Skipping commit {commit[0]}: {commit[1]}")
                            continue
                    changelog += f'- {commit[1]}\n'
        changelog += '\n'
    
    with open('CHANGELOG.md', 'w') as f:
        f.write(changelog)


def main(config_file='build_config.yaml'):
    """Generate changelog for the project
    
    Args:
        config_file: Path to configuration YAML file
    """
    try:
        with open(config_file, 'r') as f:
            config = yaml.safe_load(f)
    except Exception as e:
        print(f"Failed to load config file: {e}")
        return
    
    generate_changelog(config_file)


if __name__ == "__main__":
    # Maintain backward compatibility when run directly
    main()



#!/usr/bin/env python3
import uuid
import subprocess
import os
import re
from pathlib import Path
from typing import Dict, Any
import yaml
import sys

def build_wix_installer(config: Dict[str, Any]) -> None:
    """Build the WiX installer if enabled in config."""
    if not check_wix_installed():
        print("Error: Wix Toolset not found. Please install it from:")
        print("https://docs.firegiant.com/wix/")
        sys.exit(1)
    
    if not check_wix_extension():
        install_wix_extension()
    
    if not config.get('installer', {}).get('enabled', False):
        return
    
    installer = config['installer']
    build = config['build']
    
    # Ensure required directories exist
    output_dir = Path(installer['output']['directory'])
    output_dir.mkdir(exist_ok=True)
    
    # Check if the executable exists
    exe_path = Path(build['output']['directory']) / build['output']['filename']
    if not exe_path.exists():
        print(f"Error: Executable not found at {exe_path}")
        print("Please build the executable first.")
        return
    
    # Generate WiX source file
    wxs_content = generate_wix_source(config)
    wxs_path = output_dir / "installer.wxs"
    with open(wxs_path, 'w') as f:
        f.write(wxs_content)
    
    try:
        # Get paths from config
        ui_dir = Path(installer['ui'].get('banner_image', '.')).parent
        license_path = Path(installer['license_file']).parent
        
        # Build MSI using WiX
        cmd = [
            "wix",
            "build",
            str(wxs_path),
            "-bindpath", f"BinDir={build['output']['directory']}",
            "-bindpath", f"UiImagesDir={ui_dir}",
            "-bindpath", f"LicenseDir={license_path}",
            "-ext", "WixToolset.UI.wixext",
            "-o", str(output_dir / installer['output']['filename'])
        ]
            
        subprocess.run(cmd, check=True)
        print("WiX installer built successfully!")
        
    except subprocess.CalledProcessError as e:
        print(f"Error building installer: {e}")
        raise

def generate_wix_source(config: Dict[str, Any]) -> str:
    """Generate WiX source file content."""
    installer = config['installer']
    project = config['project']
    build = config['build']
    build_output_dir = Path(build['output']['directory'])

    def sanitize_wix_id(name: str) -> str:
        """Replace invalid WiX ID characters with underscores, ensure valid start, and limit length."""
        sanitized = re.sub(r'[^a-zA-Z0-9.]+', '_', name)
        sanitized = sanitized.strip('_')
        if not sanitized or not re.match(r'^[a-zA-Z_]', sanitized):
            sanitized = '_' + sanitized
        return sanitized[:70]

    # --- Generate XML for items to copy beside ---
    copy_beside_items = build.get('copy_beside', [])
    # Placeholders for generated XML parts
    copy_beside_dir_and_comp_xml = "" # Will hold combined <Directory><Component>...</Component></Directory> structure
    copy_beside_component_refs_xml = "" # For <Feature>

    for item_name in copy_beside_items:
        item_source_path_on_disk = build_output_dir / item_name
        component_id = sanitize_wix_id(f"Comp_{item_name}")
        base_directory_id = sanitize_wix_id(f"Dir_{item_name}")

        if not item_source_path_on_disk.exists():
            print(f"Warning (WiX): Source item '{item_name}' not found in build output dir '{build_output_dir}', skipping.")
            continue

        if item_source_path_on_disk.is_dir():
            # 1. First, build a complete map of all directories and their IDs
            directory_id_map = {} # Maps relative paths to their WiX IDs
            
            # Add the base directory to the map
            directory_id_map[Path('.')] = base_directory_id
            
            # Walk to identify all directories first
            for root, dirs, _ in os.walk(item_source_path_on_disk):
                current_dir_path = Path(root)
                relative_dir_path = current_dir_path.relative_to(item_source_path_on_disk)
                
                # Skip the base directory (already added)
                if relative_dir_path == Path('.'):
                    continue
                    
                # Create a unique ID for this directory
                dir_rel_path_str = relative_dir_path.as_posix().replace('/', '_')
                dir_id = sanitize_wix_id(f"{base_directory_id}_{dir_rel_path_str}")
                directory_id_map[relative_dir_path] = dir_id
            
            # 2. Build the directory structure XML using a simpler approach
            # Start with the base directory
            directory_xml = f'\n  <Directory Id="{base_directory_id}" Name="{item_name}">'
            
            # Function to recursively build directory structure
            def build_directory_structure(parent_path, indent="  "):
                result = ""
                # Find all immediate children of this parent
                children = [p for p in directory_id_map.keys() 
                           if p != Path('.') and p.parent == parent_path]
                
                for child_path in sorted(children):
                    child_id = directory_id_map[child_path]
                    child_name = child_path.name
                    
                    # Start this directory element
                    result += f"\n{indent}  <Directory Id=\"{child_id}\" Name=\"{child_name}\">"
                    
                    # Recursively add its children
                    result += build_directory_structure(child_path, indent + "  ")
                    
                    # Close this directory element
                    result += f"\n{indent}  </Directory>"
                
                return result
            
            # Build the nested structure starting from the root
            directory_xml += build_directory_structure(Path('.'))
            
            # 3. Create components for each directory with its files
            components_xml = ""
            component_refs_xml = ""
            
            # Walk the directory structure again to create components for each directory with files
            for root, _, files in os.walk(item_source_path_on_disk):
                # Skip if no files in this directory
                if not files:
                    continue
                    
                current_dir_path = Path(root)
                relative_dir_path = current_dir_path.relative_to(item_source_path_on_disk)
                
                # Get the directory ID from our map
                dir_id = directory_id_map.get(relative_dir_path, base_directory_id)
                
                # Create a unique component ID for this directory
                dir_component_id = sanitize_wix_id(f"Comp_{item_name}_{relative_dir_path.as_posix().replace('/', '_')}")
                
                # Build file elements for this directory
                files_xml = ""
                for file in files:
                    file_id = sanitize_wix_id(f"File_{dir_component_id}_{file}")
                    # Source path relative to BinDir
                    file_source_rel_bindir = Path(item_name) / relative_dir_path / file
                    files_xml += f'''
          <File Id="{file_id}" Name="{file}" Source="!(bindpath.BinDir)\\{file_source_rel_bindir.as_posix()}" />'''
                
                # Create a component for this directory and its files
                create_folder_xml = '\n      <CreateFolder/>' if relative_dir_path == Path('.') else ''
                remove_folder_xml = ''
                if relative_dir_path == Path('.'):
                    # Only add RemoveFolder for the base directory
                    remove_folder_id = sanitize_wix_id(f'Remove_{dir_id}')
                    remove_folder_xml = f'\n      <RemoveFolder Id="{remove_folder_id}" On="uninstall"/>'
                
                # Create the component definition
                component_xml = f'''
    <Component Id="{dir_component_id}" Guid="{str(uuid.uuid4())}" Directory="{dir_id}">{create_folder_xml}{files_xml}{remove_folder_xml}
      <RegistryValue Root="HKCU" Key="Software\\{installer['metadata']['manufacturer']}\\{installer['metadata']['product_name']}\\{dir_component_id}" Name="installed" Type="integer" Value="1" KeyPath="yes" />
    </Component>'''
                
                components_xml += component_xml
                component_refs_xml += f'\n            <ComponentRef Id="{dir_component_id}" />'
            
            # 4. Combine everything - first the directory structure, then components
            directory_xml += components_xml
            
            # Close the base directory element
            directory_xml += '\n  </Directory>'
            
            # Append the complete XML for this item
            copy_beside_dir_and_comp_xml += directory_xml
            # Add the component references for the Feature list
            copy_beside_component_refs_xml += component_refs_xml

        elif item_source_path_on_disk.is_file():
            # Generate component for a single file directly under INSTALLFOLDER
            file_id = sanitize_wix_id(f"File_{component_id}_{item_name}")
            file_source_rel_bindir = Path(item_name)
            # Note: Single files don't need a separate <Directory> element, component goes direct in INSTALLFOLDER
            single_file_component = f'''
    <Component Id="{component_id}" Guid="{str(uuid.uuid4())}" Directory="INSTALLFOLDER">
        <File Id="{file_id}"
              Name="{item_name}"
              Source="!(bindpath.BinDir)\\{file_source_rel_bindir.as_posix()}"
              KeyPath="yes"/>
    </Component>'''
            # Add the component XML directly (no surrounding Directory needed here)
            copy_beside_dir_and_comp_xml += single_file_component
            # Add component ref
            copy_beside_component_refs_xml += f'\n            <ComponentRef Id="{component_id}" />'

    # --- End generate XML ---

    # Only include URL properties if they are set
    url_properties = ""
    if project.get('url'):
        url_properties = f'''
        <Property Id="ARPURLINFOABOUT" Value="{project['url']}" />
        <Property Id="ARPHELPLINK" Value="{project['url']}" />'''

    # Get relative paths for resources
    banner_path = Path(installer['ui']['banner_image']).name
    dialog_path = Path(installer['ui']['dialog_image']).name
    license_path = Path(installer['license_file']).name

    return f'''<?xml version="1.0" encoding="UTF-8"?>
<Wix xmlns="http://wixtoolset.org/schemas/v4/wxs"
     xmlns:ui="http://wixtoolset.org/schemas/v4/wxs/ui"
     xmlns:util="http://wixtoolset.org/schemas/v4/wxs/util">
    <Package
        Name="{installer['metadata']['product_name']}"
        Manufacturer="{installer['metadata']['manufacturer']}"
        Version="{project['version']}"
        UpgradeCode="{installer['metadata']['upgrade_code']}"
        Scope="perMachine">
        
        <MajorUpgrade DowngradeErrorMessage="A newer version of [ProductName] is already installed." />
        <MediaTemplate EmbedCab="yes" />
        
        <!-- Application Icon -->
        {f'<Icon Id="app.ico" SourceFile="{project["icon"]}" />' if project.get('icon') else ''}
        {f'<Property Id="ARPPRODUCTICON" Value="app.ico" />' if project.get('icon') else ''}
        
        <!-- Add/Remove Programs Information -->{url_properties}
        <Property Id="ARPCOMMENTS" Value="{project['description']}" />
        <Property Id="ARPCONTACT" Value="{installer['metadata']['manufacturer']}" />
        <Property Id="ARPNOREPAIR" Value="1" />
        
        <!-- Directory Structure -->
        <StandardDirectory Id="ProgramFiles64Folder">
            <Directory Id="INSTALLFOLDER" Name="{installer['metadata']['product_name']}">
                <Component Id="MainExecutable" Guid="{str(uuid.uuid4())}">
                    <File Id="MainEXE"
                          Name="{build['output']['filename']}"
                          Source="!(bindpath.BinDir)\\{build['output']['filename']}"
                          KeyPath="yes">
                        <!-- Grant read and execute permissions to all users -->
                        <Permission User="Everyone" GenericAll="yes" />
                    </File>
                    
                    <!-- Register application -->
                    <RegistryValue Root="HKLM"
                                 Key="Software\\{installer['metadata']['product_name']}"
                                 Name="InstallPath"
                                 Type="string"
                                 Value="[INSTALLFOLDER]" />
                </Component>

                {copy_beside_dir_and_comp_xml}
            </Directory>
        </StandardDirectory>
        
        <!-- Start Menu -->
        <StandardDirectory Id="ProgramMenuFolder">
            <Directory Id="ApplicationProgramsFolder" Name="{installer['metadata']['product_name']}">
                <Component Id="ApplicationShortcuts" Guid="{str(uuid.uuid4())}">
                    <Shortcut Id="ApplicationShortcut"
                             Name="{installer['metadata']['product_name']}"
                             Description="{project['description']}"
                             Target="[INSTALLFOLDER]{build['output']['filename']}"
                             WorkingDirectory="INSTALLFOLDER"
                             Icon="app.ico" />
                    <Shortcut Id="UninstallProduct"
                             Name="Uninstall {installer['metadata']['product_name']}"
                             Description="Uninstall {installer['metadata']['product_name']}"
                             Target="[SystemFolder]msiexec.exe"
                             Arguments="/x [ProductCode]"/>
                    <RemoveFolder Id="CleanUpShortCut" Directory="ApplicationProgramsFolder" On="uninstall" />
                    <RegistryValue Root="HKCU"
                                 Key="Software\\{installer['metadata']['manufacturer']}\\{installer['metadata']['product_name']}"
                                 Name="installed"
                                 Type="integer"
                                 Value="1"
                                 KeyPath="yes" />
                </Component>
            </Directory>
        </StandardDirectory>
        
        <!-- Features -->
        <Feature Id="ProductFeature" Title="{installer['metadata']['product_name']}" Level="1">
            <ComponentRef Id="MainExecutable" />
            <ComponentRef Id="ApplicationShortcuts" />
            {copy_beside_component_refs_xml}
        </Feature>
        
        <!-- UI -->
        <Property Id="WIXUI_INSTALLDIR" Value="INSTALLFOLDER" />
        <Property Id="WIXUI_EXITDIALOGOPTIONALTEXT" Value="Thank you for installing {installer['metadata']['product_name']}." />
        
        <!-- Custom UI Images -->
        <WixVariable Id="WixUIDialogBmp" Value="!(bindpath.UiImagesDir)\\{dialog_path}" />
        <WixVariable Id="WixUIBannerBmp" Value="!(bindpath.UiImagesDir)\\{banner_path}" />

        <!-- License -->
        <WixVariable Id="WixUILicenseRtf" Value="!(bindpath.LicenseDir)\\{license_path}" />
        
        <ui:WixUI Id="WixUI_InstallDir" />
        
    </Package>
</Wix>'''

def check_wix_installed() -> bool:
    """Check if Wix Toolset is installed."""
    try:
        subprocess.run(['wix', '--version'], capture_output=True, check=True)
        return True
    except (subprocess.CalledProcessError, FileNotFoundError):
        return False

def check_wix_extension() -> bool:
    """Check if Wix UI extension is installed."""
    try:
        result = subprocess.run(['wix', 'extension', 'list'], capture_output=True, text=True, check=True)
        return 'WixToolset.UI.wixext' in result.stdout
    except subprocess.CalledProcessError:
        return False


def install_wix_extension() -> None:
    """Install Wix UI extension if missing."""
    try:
        print("Installing Wix UI extension...")
        subprocess.run(['wix', 'extension', 'add', 'WixToolset.UI.wixext'], check=True)
        print("Extension installed successfully")
    except subprocess.CalledProcessError as e:
        print(f"Failed to install Wix UI extension: {e}")
        sys.exit(1)


def main(config_file='build_config.yaml'):
    """Create Windows installer using Wix
    
    Args:
        config_file: Path to configuration YAML file
    """
    try:
        with open(config_file, 'r') as f:
            config = yaml.safe_load(f)
    except FileNotFoundError:
        print(f"Error: Configuration file '{config_file}' not found.")
        return
    
    build_wix_installer(config)

if __name__ == "__main__":
    # Maintain backward compatibility when run directly
    main()


#!/usr/bin/env python3
import os
import sys
import subprocess
import shutil
import yaml
from pathlib import Path
from typing import Dict, Any
from .build_installer import build_wix_installer

def find_venv_python() -> str:
    """Find the Python interpreter in the virtual environment."""
    venv_dirs = ['.venv', 'venv', 'env']
    python_exe = "python.exe" if sys.platform == "win32" else "python"
    python_path = os.path.join("Scripts" if sys.platform == "win32" else "bin", python_exe)
    
    current_dir = Path.cwd()
    for venv_dir in venv_dirs:
        # Check current directory
        venv_python = current_dir / venv_dir / python_path
        if venv_python.exists():
            return str(venv_python)
        
        # Check parent directory
        parent_venv_python = current_dir.parent / venv_dir / python_path
        if parent_venv_python.exists():
            return str(parent_venv_python)
    
    return sys.executable

def load_config(config_file: str = 'build_config.yaml') -> Dict[str, Any]:
    """Load and validate the build configuration."""
    with open(config_file, 'r') as f:
        config = yaml.safe_load(f)
    
    required_keys = ['project', 'build']
    for key in required_keys:
        if key not in config:
            raise ValueError(f"Missing required key '{key}' in config file")
    
    return config

def build_nuitka_command(config: Dict[str, Any], python_exe: str) -> list:
    """Construct the Nuitka command from configuration."""
    project = config['project']
    build = config['build']
    main_file = project['main_file']
    
    cmd = [
        python_exe,
        "-m",
        "nuitka",
        main_file,
    ]

    # Basic options
    if build['options']['standalone']:
        cmd.append("--standalone")
    if build['options']['onefile']:
        cmd.append("--onefile")

    # Project metadata
    if project.get('icon'):
        cmd.append(f"--windows-icon-from-ico={project['icon']}")
    cmd.extend([
        f"--company-name={project['company']}",
        f"--product-name={project['name']}",
        f"--product-version={project['version']}",
        f"--file-description={project['description']}"
    ])

    # Splash screen
    if build['options'].get('splash_screen') and sys.platform == "win32":
        cmd.append(f"--onefile-windows-splash-screen-image={build['options']['splash_screen']}")

    # Distribution metadata
    for meta in build['include'].get('distribution_metadata', []):
        cmd.append(f"--include-distribution-metadata={meta}")

    # Packages
    for package in build['include'].get('packages', []):
        cmd.append(f"--include-package={package}")
        if package == "PySide6":
            cmd.append("--enable-plugin=pyside6")

    # Data directories
    for data_dir in build['include'].get('data_dirs', []):
        source_path = data_dir['source']
        target_path = data_dir['target']
        cmd.append(f"--include-data-dir={source_path}={target_path}")

    # External data
    for data in build['include'].get('external_data', []):
        cmd.append(f"--include-onefile-external-data={data}")

    # External files
    for file in build['include'].get('files', []):
        cmd.append(f"--include-data-file={file}={file}")

    # Remove .exe from output filename if we are not on Windows
    if sys.platform == "win32":
        file_name = build['output']['filename']
    else:
        file_name = build['output']['filename'][:-4]
    # Output settings
    cmd.extend([
        f"--output-dir={build['output']['directory']}",
        f"--output-filename={file_name}"
    ])

    # Debug settings
    debug = config.get('debug', {})
    if debug.get('enabled', False):
        cmd.extend([
            "--windows-console-mode=force",
            "--force-stdout-spec={PROGRAM_BASE}.out.txt",
            "--force-stderr-spec={PROGRAM_BASE}.err.txt"
        ])
    else:
        cmd.append("--windows-console-mode=disable")

    return cmd

def main(config_file='build_config.yaml'):
    """Build executable with Nuitka
    
    Args:
        config_file: Path to configuration YAML file
    """
    try:
        with open(config_file, 'r') as f:
            config = yaml.safe_load(f)
        
        # Find virtual environment Python
        python_exe = find_venv_python()
        
        # Build the Nuitka command
        cmd = build_nuitka_command(config, python_exe)
        
        print("Building executable with Nuitka...")
        subprocess.run(cmd, check=True)
        print("Executable built successfully!")

        # --- Copy files/folders beside the executable ---
        build_config = config.get('build', {})
        copy_beside_items = build_config.get('copy_beside', [])
        if copy_beside_items:
            output_dir_name = build_config.get('output', {}).get('directory', 'dist')
            output_dir_path = Path.cwd() / output_dir_name
            
            print(f"\nCopying additional items to {output_dir_path}...")
            
            for item_name in copy_beside_items:
                source_path = Path.cwd() / item_name
                dest_path = output_dir_path / item_name
                
                if not source_path.exists():
                    print(f"Warning: Source item '{item_name}' not found at '{source_path}', skipping.")
                    continue
                
                try:
                    if source_path.is_dir():
                        print(f"  Copying directory: {item_name}")
                        shutil.copytree(source_path, dest_path, dirs_exist_ok=True)
                    elif source_path.is_file():
                        print(f"  Copying file: {item_name}")
                        # Ensure the destination directory exists (should already exist)
                        output_dir_path.mkdir(parents=True, exist_ok=True)
                        shutil.copy2(source_path, dest_path)
                    else:
                        print(f"Warning: Source item '{item_name}' is not a file or directory, skipping.")
                except Exception as copy_e:
                    print(f"Error copying '{item_name}': {copy_e}")
            print("Finished copying additional items.")
        # --- End copy section ---

        
        # Build installer if enabled
        if config.get('installer', {}).get('enabled', False):
            print("\nBuilding WiX installer...")
            build_wix_installer(config)
    
    except Exception as e:
        print(f"Error: {str(e)}")
        sys.exit(1)

if __name__ == "__main__":
    # Maintain backward compatibility when run directly
    main()



import argparse
from . import build_exe
from . import build_installer
from . import changelog_generator

def main():
    parser = argparse.ArgumentParser(description='Python Installer Creator Tool')
    subparsers = parser.add_subparsers(dest='command', required=True)
    
    # Build EXE command
    build_exe_parser = subparsers.add_parser('build-exe', help='Build executable with Nuitka')
    build_exe_parser.add_argument('-c', '--config', default='build_config.yaml', help='Configuration file')
    
    # Build Installer command
    build_installer_parser = subparsers.add_parser('build-installer', help='Create Windows installer')
    build_installer_parser.add_argument('-c', '--config', default='build_config.yaml', help='Configuration file')
    
    # Generate Changelog command
    subparsers.add_parser('generate-changelog', help='Generate changelog')
    
    # Generate UUID command
    uuid_parser = subparsers.add_parser('generate-uuid', help='Generate UUID for upgrade codes')
    uuid_parser.add_argument('-s', '--string', help='String to generate deterministic UUID from')
    uuid_parser.add_argument('-r', '--random', action='store_true', help='Generate random UUID')
    
    args = parser.parse_args()
    
    if args.command == 'build-exe':
        build_exe.main(config_file=args.config)
    elif args.command == 'build-installer':
        build_installer.main(config_file=args.config)
    elif args.command == 'generate-changelog':
        changelog_generator.main()
    elif args.command == 'generate-uuid':
        import uuid
        if args.string:
            # Generate deterministic UUID from string (version 5)
            namespace = uuid.NAMESPACE_DNS
            print(uuid.uuid5(namespace, args.string))
        elif args.random:
            # Generate random UUID (version 4)
            print(uuid.uuid4())
        else:
            print('Please specify either --string or --random')
